%
%
%  Filaments Branch Hierarchy for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Filaments Functions">
%        <Item name="Filaments Branch Hierarchy" icon="Matlab">
%          <Command>MatlabXT::XTFilamentsBranchHierarchy(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpFilaments">
%          <Item name="Branch Hierarchy" icon="Matlab">
%            <Command>MatlabXT::XTFilamentsBranchHierarchy(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%
%  Description:
%
%   Split one filament object in different filaments. 
%   The branches directly connected to the root are first-level branches; 
%       The branches directly connected to the first-level branches are 
%       second-level branches; and so on. 
%   For each level, a new filament object, which collects the branches of
%       the level, is created. 
%   The number of filaments created is equal to the maximal level of the
%       branches of the original filament.
%
%

function XTFilamentsBranchHierarchy(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

% get the filament
vFactory = vImarisApplication.GetFactory;
vFilaments = vFactory.ToFilaments(vImarisApplication.GetSurpassSelection);

% search the filament if not previously selected
vSurpassScene = vImarisApplication.GetSurpassScene;
if ~vFactory.IsFilaments(vFilaments)        
    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
        if vFactory.IsFilaments(vDataItem)
            vFilaments = vFactory.ToFilaments(vDataItem);
            break;
        end
    end
    % did we find the filament?
    if isequal(vFilaments, [])
        msgbox('Please create some filament!');
        return;
    end
end

vProgressDisplay = waitbar(0, 'Reading Filaments statistics');

vStatistics = vFilaments.GetStatistics;
vNames = cell(vStatistics.mNames);
vValues = vStatistics.mValues;
vIds = vStatistics.mIds;

vDendriteIndices = strmatch('Dendrite Branch Level', vNames);
vSpineIndices = strmatch('Spine Branch Level', vNames);
vValidIndices = [vDendriteIndices; vSpineIndices];
if isempty(vValidIndices)
  close(vProgressDisplay)
  msgbox('Please enable "Dendrite Branch Level" and "Spine Branch Level" statistics!')
  return
end

vLevelValues = vValues(vValidIndices);
vLevelIds = vIds(vValidIndices);

waitbar(0, vProgressDisplay, 'Creating new Filaments');

% get the jet colormap
vColorMap = jet;
vColorMapSize = size(vColorMap, 1);
% close the figure opened by jet
close

vCount = vFilaments.GetNumberOfFilaments;
vNumberOfLevels = max(vLevelValues);
vNewFilaments = cell(vNumberOfLevels, 1);
vThisLevelIds = cell(vNumberOfLevels, 1);
for vBranchLevel = 1:vNumberOfLevels
  vNewFilament = vImarisApplication.GetFactory.CreateFilaments;
  vNewFilament.SetName(sprintf('%s [level %i]', char(vFilaments.GetName), vBranchLevel));
  % computes the color
  vPercent = (vBranchLevel-1)/max(vNumberOfLevels-1,1)*(vColorMapSize-1)+1;
  vColor = interp1(vColorMap, vPercent);
  vNewFilament.SetColorRGBA(uint32(round(vColor * 255) * [1; 256; 256*256]));
  vNewFilament.SetColorSpinesRGBA(uint32(round((vColor+1)/2 * 255) * [1; 256; 256*256]));
  vNewFilaments{vBranchLevel} = vNewFilament;
  vThisLevelIds{vBranchLevel} = vLevelIds(vLevelValues == vBranchLevel);  
end

waitbar(0, vProgressDisplay, 'Splitting Filaments');

for vFilamentIndex = 0:vCount-1
  vFilamentsXYZ = vFilaments.GetPositionsXYZ(vFilamentIndex);
  vFilamentsEdges = vFilaments.GetEdges(vFilamentIndex) + 1;
  vFilamentsRadius = vFilaments.GetRadii(vFilamentIndex);
  vFilamentsTypes = vFilaments.GetTypes(vFilamentIndex);
  vFilamentsTime = vFilaments.GetTimeIndex(vFilamentIndex);
  % vBeginningVertex = vFilaments.GetBeginningVertexIndex(vFilamentIndex) + 1;
  vEdgesSegmentIds = vFilaments.GetEdgesSegmentId(vFilamentIndex);
  for vBranchLevel = 1:vNumberOfLevels
    vEdgesThisLevel = false(numel(vEdgesSegmentIds), 1);
    vThisBranchLevelIds = vThisLevelIds{vBranchLevel}';
    for vId = vThisBranchLevelIds
      vEdgesThisLevel(vEdgesSegmentIds == vId) = true;
    end
    
    % remap points and edges
    vEdges = vFilamentsEdges(vEdgesThisLevel, :);
    vThisPoints = unique(vEdges(:));
    if ~isempty(vThisPoints)
      vIndices = zeros(max(vThisPoints), 1) - 1;
      vIndices(vThisPoints) = 0:numel(vThisPoints) - 1;
      vThisEdges = vIndices(vEdges);

      vNewFilaments{vBranchLevel}.AddFilament( ...
        vFilamentsXYZ(vThisPoints, :), vFilamentsRadius(vThisPoints), ...
        vFilamentsTypes(vThisPoints), vThisEdges, vFilamentsTime);
    end  
    waitbar((vFilamentIndex+vBranchLevel/vNumberOfLevels)/vCount, vProgressDisplay);
  end
end

waitbar(1, vProgressDisplay, 'Adding new Filaments to Imaris Scene');

vParent = vImarisApplication.GetFactory.CreateDataContainer;
vParent.SetName(sprintf('%s Branch Levels', char(vFilaments.GetName)));
vFilaments.GetParent.AddChild(vParent, -1);
vFilaments.SetVisible(0);
for vBranchLevel = 1:vNumberOfLevels
  vParent.AddChild(vNewFilaments{vBranchLevel}, -1);
end

close(vProgressDisplay);

